//
// ABM.java
//

import java.util.*;

public class ABM
{
    public static final int NUM = 10;
    public static Random RNG;
    public static final long RNG_SEED = 12345;
    public Student students[];
    public int presList[];

    ABM()
    {
	super();

	// create RNG
	//RNG = new Random(RNG_SEED);
	RNG = new Random();

	// presentation order
	presList = new int[NUM];
	for (int i = 0; i < NUM; ++i)
	    presList[i] = -1;
    }

    // create the set of students
    public void createStudents()
    {
	students = new Student[NUM];
	for (int i = 0; i < NUM; ++i) {
	    students[i] = new Student(i);
	    students[i].probPreference();
	}
    }

    // 
    public void runAlg()
    {
	boolean done = false;
	int rounds = 0;

	while (!done) {
	    // randomly pick two agents to interact
	    int a1 = RNG.nextInt(NUM);
	    int a2 = RNG.nextInt(NUM);
	    if (a1 == a2) continue;
	    students[a1].interactWith(students[a2]);
	    ++rounds;

	    int cLevel = consensusLevel();
	    if (cLevel == 100) done = true;
	}
	//System.out.println("Total rounds: " + rounds);
    }

    public int consensusLevel()
    {
	int cnt[] = new int[NUM];
	int cLevel = 0;

	// for each position
	// determine majority agreement
	for (int i = 0; i < NUM; ++i) {
	    for (int j = 0; j < NUM; ++j)
		cnt[j] = 0;

	    for (int j = 0; j < NUM; ++j) {
		int[] pl = students[j].getPresList();
		if (pl[i] == -1) continue;
		++cnt[pl[i]];
	    }

	    int max = -1;
	    for (int j = 0; j < NUM; ++j) {
		if (cnt[j] > max)
		    max = cnt[j];
	    }

	    cLevel += max;
	}
	//System.out.println("Consensus Level: " + cLevel);
	//printStudentPref();

	return cLevel;
    }

    public void printStudentPref()
    {
	System.out.println("Student lists.");
	for (int i = 0; i < NUM; ++i) {
	    System.out.print("Student " + i + ":");
	    students[i].printList();
	    System.out.println("");
	}
    }

    public void printList()
    {
	System.out.println("Order of presentations is:");
	for (int i = 0; i < NUM; ++i)
	    System.out.println("Student " + presList[i]);
    }

    public void calcMetric()
    {
	int metric = 0;
	for (int i = 0; i < NUM; ++i) {
	    metric += students[i].getMetric();
	}
	//System.out.println("Metric: " + metric);
	System.out.println(metric);
    }

    public static void main(String args[])
    {
	for (int i = 0; i < 100; ++i) {
	    ABM ma = new ABM();

	    ma.createStudents();
	    //ma.printStudentPref();
	    ma.runAlg();
	    //ma.printStudentPref();
	    ma.calcMetric();
	}
    }
}
